@@ -17,10 +17,11 @@ sub ProcessUnknown($$$);
 sub ProcessUnknownOrPreview($$$);
 sub ProcessCanon($$$);
 sub ProcessGE2($$$);
+sub ProcessKodakPatch($$$);
 sub WriteUnknownOrPreview($$$);
 sub FixLeicaBase($$;$);
 
-$VERSION = '1.87';
+$VERSION = '1.93';
 
 my $debug;          # set to 1 to enable debugging code
 
@@ -352,6 +353,24 @@ my $debug;          # set to 1 to enable debugging code
     },
     {
         Name => 'MakerNoteKodak8b',
+        # these maker notes have an extra 2 bytes after the entry count
+        # (this is handled by the patch).  Also, the IFD uses a Format 13,
+        # which is some 2-byte format (not Float, as decoded by ExifTool)
+        # - written by the PixPro AZ251, AZ361, AZ262, AZ521
+        Condition => q{
+            $$self{Make}=~/Kodak/i and
+            $$valPt =~ /^MM\0\x2a\0\0\0\x08\0.\0\0/
+        },
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Kodak::Type8',
+            ProcessProc => \&ProcessKodakPatch,
+            ByteOrder => 'BigEndian',
+            Start => '$valuePtr + 8',
+            Base => '$start - 8',
+        },
+    },
+    {
+        Name => 'MakerNoteKodak8c',
         # TIFF-format maker notes
         Condition => q{
             $$self{Make}=~/Kodak/i and
@@ -391,6 +410,22 @@ my $debug;          # set to 1 to enable debugging code
             Start => '$valuePtr + 2',
         },
     },
+    {
+        Name => 'MakerNoteKodak11',
+        # these maker notes have an extra 2 bytes after the entry count
+        # - written by the PixPro S-1 (Note: Make is "JK Imaging, Ltd.", so check Model for "Kodak")
+        Condition => q{
+            $$self{Model}=~/Kodak/i and
+            $$valPt =~ /^II\x2a\0\x08\0\0\0.\0\0\0/
+        },
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Kodak::Type11',
+            ProcessProc => \&ProcessKodakPatch,
+            ByteOrder => 'LittleEndian',
+            Start => '$valuePtr + 8',
+            Base => '$start - 8',
+        },
+    },
     {
         Name => 'MakerNoteKodakUnknown',
         Condition => '$$self{Make}=~/Kodak/i and $$valPt!~/^AOC\0/',
@@ -464,6 +499,15 @@ my $debug;          # set to 1 to enable debugging code
             ByteOrder => 'Unknown', # most are little-endian, but D1 is big
         },
     },
+    {
+        Name => 'MakerNoteNintendo',
+        # (starts with an IFD)
+        Condition => '$$self{Make} eq "Nintendo"',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Nintendo::Main',
+            ByteOrder => 'Unknown',
+        },
+    },
     {
         Name => 'MakerNoteOlympus',
         # (if Make is 'SEIKO EPSON CORP.', starts with "EPSON\0")
@@ -538,11 +582,12 @@ my $debug;          # set to 1 to enable debugging code
         },
     },
     {
-        Name => 'MakerNoteLeica5', # used by the X1/X2
+        Name => 'MakerNoteLeica5', # used by the X1/X2/X VARIO/T
         # (X1 starts with "LEICA\0\x01\0", Make is "LEICA CAMERA AG")
         # (X2 starts with "LEICA\0\x05\0", Make is "LEICA CAMERA AG")
         # (X VARIO starts with "LEICA\0\x04\0", Make is "LEICA CAMERA AG")
-        Condition => '$$valPt =~ /^LEICA\0[\x01\x04\x05]\0/',
+        # (T (Typ 701) starts with LEICA\0\0x6", Make is "LEICA CAMERA AG")
+        Condition => '$$valPt =~ /^LEICA\0[\x01\x04\x05\x06]\0/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Panasonic::Leica5',
             Start => '$valuePtr + 8',
@@ -551,10 +596,15 @@ my $debug;          # set to 1 to enable debugging code
         },
     },
     {
-        Name => 'MakerNoteLeica6', # used by the S2 and M (Typ 240)
-        # (starts with "LEICA\0\x02\xff", Make is "Leica Camera AG",
-        #  but maker notes aren't loaded at the time this is tested)
-        Condition => '$$self{Model} eq "S2" or $$self{Model} eq "LEICA M (Typ 240)"',
+        Name => 'MakerNoteLeica6', # used by the S2, M (Typ 240) and S (Typ 006)
+        # (starts with "LEICA\0\x02\xff", Make is "Leica Camera AG", but test the
+        # model names separately because the maker notes data may not be loaded
+        # at the time this is tested if they are in a JPEG trailer)
+        Condition => q{
+            ($$self{Make} eq 'Leica Camera AG' and ($$self{Model} eq "S2" or
+            $$self{Model} eq "LEICA M (Typ 240)" or $$self{Model} eq "LEICA S (Typ 006)")) or
+            $$valPt =~ /^LEICA\0\x02\xff/
+        },
         DataTag => 'LeicaTrailer',  # (generates fixup name for this tag)
         LeicaTrailer => 1, # flag to special-case this tag in the Exif code
         SubDirectory => {
@@ -693,15 +743,35 @@ my $debug;          # set to 1 to enable debugging code
     {
         Name => 'MakerNoteRicoh',
         # (my test R50 image starts with "      \x02\x01" - PH)
-        # (the HZ15 starts with "MM\0\x2a" but an extra 2 bytes of padding after
-        #  the IFD entry count prevents these from being processed as a standard IFD)
-        Condition => '$$self{Make}=~/^(PENTAX )?RICOH/ and $$valPt=~/^(Ricoh|      |MM\0\x2a|II\x2a\0)/i',
+        Condition => q{
+            $$self{Make} =~ /^(PENTAX )?RICOH/ and
+            $$valPt =~ /^(Ricoh|      |MM\0\x2a|II\x2a\0)/i and
+            $$valPt !~ /^(MM\0\x2a\0\0\0\x08\0.\0\0|II\x2a\0\x08\0\0\0.\0\0\0)/s
+        },
         SubDirectory => {
             TagTable => 'Image::ExifTool::Ricoh::Main',
             Start => '$valuePtr + 8',
             ByteOrder => 'Unknown',
         },
     },
+    {
+        Name => 'MakerNoteRicoh2',
+        # (the Ricoh HZ15 starts with "MM\0\x2a" and the Pentax XG-1 starts with "II\x2a\0",
+        # but an extra 2 bytes of padding after the IFD entry count prevents these from
+        # being processed as a standard IFD.  Note that the offsets for the HZ15 are all
+        # zeros, but they seem to be mostly OK for the XG-1)
+        Condition => q{
+            $$self{Make} =~ /^(PENTAX )?RICOH/ and
+            $$valPt =~ /^(MM\0\x2a\0\0\0\x08\0.\0\0|II\x2a\0\x08\0\0\0.\0\0\0)/s
+        },
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Ricoh::Type2',
+            Start => '$valuePtr + 8',
+            Base => '$start - 8',
+            ByteOrder => 'Unknown',
+            ProcessProc => \&ProcessKodakPatch,
+        },
+    },
     {
         Name => 'MakerNoteRicohText',
         Condition => '$$self{Make}=~/^RICOH/',
@@ -723,7 +793,7 @@ my $debug;          # set to 1 to enable debugging code
         # Samsung STMN maker notes WITH PreviewImage
         Condition => '$$valPt =~ /^STMN\d{3}/',
         SubDirectory => {
-            TagTable => 'Image::ExifTool::Samsung::Type1',
+            TagTable => 'Image::ExifTool::Samsung::Main',
         },
     },
     {
@@ -792,7 +862,8 @@ my $debug;          # set to 1 to enable debugging code
         Name => 'MakerNoteSony',
         # (starts with "SONY DSC \0" or "SONY CAM \0")
         # (TF1 starts with "\0\0SONY PIC\0")
-        Condition => '$$valPt=~/^SONY (DSC|CAM)/ or $$valPt=~/^\0\0SONY PIC\0/',
+        # (Hasselblad models start with "VHAB     \0")
+        Condition => '$$valPt=~/^(SONY (DSC|CAM)|\0\0SONY PIC\0|VHAB     \0)/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Sony::Main',
             Start => '$valuePtr + 12',
@@ -1249,7 +1320,7 @@ sub LocateIFD($$)
     my $dirStart = $$dirInfo{DirStart} || 0;
     # (ignore MakerNotes DirLen since sometimes this is incorrect)
     my $size = $$dirInfo{DataLen} - $dirStart;
-    my $dirLen = $$dirInfo{DirLen} || $size;
+    my $dirLen = defined $$dirInfo{DirLen} ? $$dirInfo{DirLen} : $size;
     my $tagInfo = $$dirInfo{TagInfo};
     my $ifdOffsetPos;
     # the IFD should be within the first 32 bytes
@@ -1484,6 +1555,23 @@ sub ProcessGE2($$$)
     return Image::ExifTool::Exif::ProcessExif($et, $dirInfo, $tagTablePtr);
 }
 
+#------------------------------------------------------------------------------
+# Process broken Kodak type 8b maker notes
+# Inputs: 0) ExifTool object ref, 1) DirInfo ref, 2) tag table ref
+# Returns: 1 on success
+sub ProcessKodakPatch($$$)
+{
+    my ($et, $dirInfo, $tagTablePtr) = @_;
+    my $dataPt = $$dirInfo{DataPt} or return 0;
+    my $dirStart = $$dirInfo{DirStart} || 0;
+
+    # these maker notes have 2 extra bytes after the entry count, so fix this
+    return 0 unless $$dirInfo{DirLen} > 2;
+    Set16u(Get16u($dataPt,$dirStart), $dataPt, $dirStart+2);
+    $$dirInfo{DirStart} += 2;
+    return Image::ExifTool::Exif::ProcessExif($et, $dirInfo, $tagTablePtr);
+}
+
 #------------------------------------------------------------------------------
 # Process unknown maker notes or PreviewImage
 # Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
